--- loncom/interface/lonrequestcourse.pm 2014/01/05 10:55:35 1.75
+++ loncom/interface/lonrequestcourse.pm 2014/05/09 17:43:13 1.82
@@ -1,7 +1,7 @@
# The LearningOnline Network
# Request a course
#
-# $Id: lonrequestcourse.pm,v 1.75 2014/01/05 10:55:35 raeburn Exp $
+# $Id: lonrequestcourse.pm,v 1.82 2014/05/09 17:43:13 raeburn Exp $
#
# Copyright Michigan State University Board of Trustees
#
@@ -1526,7 +1526,8 @@ sub print_request_form {
&Apache::lonnet::auto_possible_instcodes($dom,\@codetitles,\%cat_titles,
\%cat_order,\@code_order);
}
- my ($storeresult,$result) = &print_request_outcome($dom,\@codetitles,
+ my $lonhost = $r->dir_config('lonHostID');
+ my ($storeresult,$result) = &print_request_outcome($lonhost,$dom,\@codetitles,
\@code_order,$instcredits);
$r->print($result);
if (($storeresult eq 'ok') || ($storeresult eq 'created')) {
@@ -1549,7 +1550,7 @@ sub print_request_form {
if ($result eq 'created') {
my $role = 'au';
my $spec = "$role./$env{'form.showdom'}/";
- push(@links,&mt('Enter your authoring space with role: [_1]',
+ push(@links,&mt('Enter your Authoring Space with role: [_1]',
''.
&Apache::lonnet::plaintext($role).''));
}
@@ -1624,12 +1625,12 @@ sub print_request_form {
sub print_author_prompt {
my ($r,$action,$cnum,$showdom,$crstype,$storeresult) = @_;
- $r->print('
'.&mt('Access to authoring space').'
'.
+ $r->print('
'.&mt('Access to Authoring Space').'
'.
'
'.
&mt('Although assessment items can be created directly inside a course, such items only use part of the assessment capabilities of LON-CAPA.').
' '.
- &mt('By contrast, items created in authoring space, then imported into a course, can use all of the features of the assessment engine.').'
'.
- '
'.&mt('Request authoring space access now?').
+ &mt('By contrast, items created in Authoring Space, then imported into a course, can use all of the features of the assessment engine.').'
'.
+ '
'.&mt('Request Authoring Space access now?').
' '.
''.
(' 'x2).
@@ -3335,7 +3336,7 @@ sub display_navbuttons {
}
sub print_request_outcome {
- my ($dom,$codetitles,$code_order,$instcredits) = @_;
+ my ($lonhost,$dom,$codetitles,$code_order,$instcredits) = @_;
my ($output,$cnum,$now,$req_notifylist,$crstype,$enrollstart,$enrollend,
%sections,%crosslistings,%personnel,@baduname,@missingdom,%domconfig,
$uniquecode);
@@ -3580,13 +3581,14 @@ sub print_request_outcome {
accessend => $accessend,
personnel => \%personnel,
};
- my ($result,$output) = &process_request($dom,$cnum,$crstype,$now,$details,$instcode,
+ my ($result,$output) = &process_request($lonhost,$dom,$cnum,$crstype,$now,$details,$instcode,
$req_notifylist,\@instsections,\%domconfig);
return ($result,$output);
}
sub process_request {
- my ($dom,$cnum,$crstype,$now,$details,$instcode,$req_notifylist,$instsections,$domconfig) = @_;
+ my ($lonhost,$dom,$cnum,$crstype,$now,$details,$instcode,$req_notifylist,$instsections,
+ $domconfig) = @_;
my (@inststatuses,$storeresult,$creationresult,$output);
my $val =
&Apache::loncoursequeueadmin::get_processtype('course',$env{'user.name'},
@@ -3606,7 +3608,7 @@ sub process_request {
}
$storeresult = 'notpermitted';
} else {
- my ($disposition,$message,$reqstatus);
+ my ($disposition,$message,$reqstatus,$coursedesc);
my %reqhash = (
reqtime => $now,
crstype => $crstype,
@@ -3614,6 +3616,11 @@ sub process_request {
);
my $requestkey = $dom.'_'.$cnum;
my $validationerror;
+ my $fullname = &Apache::loncommon::plainname($env{'user.name'},
+ $env{'user.domain'});
+ if (ref($details) eq 'HASH') {
+ $coursedesc = $details->{'cdescr'};
+ }
if ($val eq 'autolimit=') {
$disposition = 'process';
} elsif ($val =~ /^autolimit=(\d+)$/) {
@@ -3621,7 +3628,7 @@ sub process_request {
$disposition = &check_autolimit($env{'user.name'},$env{'user.domain'},
$dom,$crstype,$limit,\$message);
} elsif ($val eq 'validate') {
- my ($inststatuslist,$validationchk,$validation);
+ my ($inststatuslist,$validationchk,$validation,%custominfo);
if (@inststatuses > 0) {
$inststatuslist = join(',',@inststatuses);
}
@@ -3631,10 +3638,27 @@ sub process_request {
$instseclist = join(',',@{$instsections});
}
}
+ my $preprocess = &Apache::lonnet::auto_crsreq_update($dom,$cnum,$crstype,'prevalidate',$env{'user.name'},
+ $env{'user.domain'},$fullname,$coursedesc);
+ if (ref($preprocess) eq 'HASH') {
+ if (ref($preprocess->{'formitems'}) eq 'HASH') {
+ foreach my $key (keys(%{$preprocess->{'formitems'}})) {
+ if ($preprocess->{'formitems'}->{$key} eq 'multiple') {
+ if (exists($env{'form.'.$key})) {
+ @{$custominfo{$key}} = &Apache::loncommon::get_env_multiple($env{'form.'.$key});
+ }
+ } else {
+ if (exists($env{'form.'.$key})) {
+ $custominfo{$key} = $env{'form.'.$key};
+ }
+ }
+ }
+ }
+ }
$validationchk =
&Apache::lonnet::auto_courserequest_validation($dom,
$env{'user.name'}.':'.$env{'user.domain'},$crstype,
- $inststatuslist,$instcode,$instseclist);
+ $inststatuslist,$instcode,$instseclist,\%custominfo);
if ($validationchk =~ /:/) {
($validation,$message) = split(':',$validationchk);
} else {
@@ -3651,13 +3675,8 @@ sub process_request {
}
$reqhash{'disposition'} = $disposition;
$reqstatus = $disposition;
- my ($modified,$queued,$coursedesc,%customitems);
+ my ($modified,$queued,$coursedesc,$token,%customitems);
unless ($disposition eq 'rejected') {
- if (ref($details) eq 'HASH') {
- $coursedesc = $details->{'cdescr'};
- }
- my $fullname = &Apache::loncommon::plainname($env{'user.name'},
- $env{'user.domain'});
my $inprocess = &Apache::lonnet::auto_crsreq_update($dom,$cnum,$crstype,'process',$env{'user.name'},
$env{'user.domain'},$fullname,$coursedesc);
if (ref($inprocess) eq 'HASH') {
@@ -3720,6 +3739,14 @@ sub process_request {
if ($code) {
$reqhash{'code'} = $code;
}
+ if (ref($postprocess) eq 'HASH') {
+ if (ref($postprocess->{'createdactions'}) eq 'HASH') {
+ if (ref($postprocess->{'createdactions'}{'environment'}) eq 'HASH') {
+ &Apache::loncoursequeueadmin::postprocess_crsenv($dom,$cnum,
+ $postprocess->{'createdactions'}{'environment'});
+ }
+ }
+ }
$output .= ' '.$role_result.'
'.
&mt('Your request has been placed in a queue pending administrative action.').' '.
&mt("Usually this means that your institution's information systems do not list you among the instructional personnel for this course.").' '.
&mt('The list of instructional personnel for the course will be automatically checked daily, and once you are listed the request will be processed.').
- '
';
+ '
';
+ if (ref($postprocess) eq 'HASH') {
+ if ($postprocess->{'pendingweb'}) {
+ $output .= $postprocess->{'pendingweb'};
+ } else {
+ $output .= $pending_default;
+ }
+ } else {
+ $output .= $pending_default;
+ }
} elsif ($disposition eq 'created') {
if (($code) || ((ref($postprocess) eq 'HASH') &&
((ref($postprocess->{'createdmsg'}) eq 'ARRAY') || ($postprocess->{'createdweb'})))) {
@@ -4043,7 +4113,7 @@ sub notification_information {
}
}
if (scalar(@{$addmsg}) > 0) {
- my $type = 'createdreq';
+ my $type = 'createdcrsreq';
if ($code) {
$type = 'uniquecode';
}
@@ -4059,6 +4129,53 @@ sub notification_information {
return $output;
}
+sub pending_validation_form {
+ my ($cdom,$cnum,$crstype,$now,$token,$lonhost,$cdesc) = @_;
+ my $output;
+ my %postvalues = (
+ 'owner' => $env{'user.name'}.':'.$env{'user.domain'},
+ 'course' => $cdom.'_'.$cnum,
+ 'coursetype' => $crstype,
+ );
+ my %domconfig = &Apache::lonnet::get_dom('configuration',['requestcourses'],$cdom);
+
+ if (ref($domconfig{'requestcourses'}) eq 'HASH') {
+ my ($url,$buttontext,$code,@fields);
+ if (ref($domconfig{'requestcourses'}{'validation'}) eq 'HASH') {
+ $postvalues{'description'} = $cdesc;
+ $url = $domconfig{'requestcourses'}{'validation'}{'url'};
+ if (ref($domconfig{'requestcourses'}{'validation'}{'fields'}) eq 'ARRAY') {
+ @fields = @{$domconfig{'requestcourses'}{'validation'}{'fields'}};
+ }
+ $buttontext = $domconfig{'requestcourses'}{'validation'}{'button'};
+ $output .= $domconfig{'requestcourses'}{'validation'}{'markup'};
+ if (($url =~ m{^(https?\://|/)}) && (@fields > 0)) {
+ $output .= ''."\n";
+ }
+ }
+ }
+ return $output;
+}
+
sub check_autolimit {
my ($uname,$udom,$dom,$crstype,$limit,$message) = @_;
my %crsroles = &Apache::lonnet::get_my_roles($env{'user.name'},$env{'user.domain'},
@@ -4276,25 +4393,26 @@ sub generate_date_items {
sub print_textbook_form {
my ($r,$dom,$incdoms,$domdefs,$settings,$can_request) = @_;
- my ($bookshash,%ordered);
+ my (%prefab,%ordered,%numprefab);
my $crstype = 'textbook';
#
# Retrieve list of textbook courses cloneable by user
#
- my $numbook;
- if (ref($settings) eq 'HASH') {
- $bookshash = $settings->{'textbooks'};
- if (ref($bookshash) eq 'HASH') {
- foreach my $item (keys(%{$bookshash})) {
- my ($clonedom,$clonecrs) = split(/_/,$item);
- if (ref($bookshash->{$item}) eq 'HASH') {
+ foreach my $type ('textbooks','templates') {
+ $numprefab{$type} = 0;
+ if (ref($settings) eq 'HASH') {
+ $prefab{$type} = $settings->{$type};
+ if (ref($prefab{$type}) eq 'HASH') {
+ foreach my $item (keys(%{$prefab{$type}})) {
my ($clonedom,$clonecrs) = split(/_/,$item);
- if (&Apache::loncoursequeueadmin::can_clone_course($env{'user.name'},
- $env{'user.domain'},$clonecrs,$clonedom,$crstype)) {
-
- my $num = $bookshash->{$item}{'order'};
- $ordered{$num} = $item;
- $numbook ++;
+ if (ref($prefab{$type}{$item}) eq 'HASH') {
+ if (&Apache::loncoursequeueadmin::can_clone_course($env{'user.name'},
+ $env{'user.domain'},$clonecrs,$clonedom,$crstype)) {
+
+ my $num = $prefab{$type}{$item}{'order'};
+ $ordered{$type}{$num} = $item;
+ $numprefab{$type} ++;
+ }
}
}
}
@@ -4330,7 +4448,7 @@ sub print_textbook_form {
my $numcurrent = scalar(keys(%cloneable));
- my $jscript = &textbook_request_javascript($numbook,$numcurrent);
+ my $jscript = &textbook_request_javascript(\%numprefab,$numcurrent);
my %loaditems;
$loaditems{'onload'} = 'javascript:uncheckAllRadio();';
$r->print(&header('Course Request',$jscript,\%loaditems));
@@ -4381,9 +4499,16 @@ sub print_textbook_form {
$r->print('
@@ -4276,25 +4393,26 @@ sub generate_date_items {
my $crstype = 'textbook';
#
-# Retrieve list of textbook courses cloneable by user
+# Retrieve list of prefabricated courses (textbook courses and templates) cloneable by user
#